/* billmove.h
**
**    includefile: physics of the billard system to calculate timestep
**    Copyright (C) 2001  Florian Berger
**    Email:  harpin_floh@yahoo.de,  florian.berger@jk.uni-linz.ac.at
**
**    This program is free software; you can redistribute it and/or modify
**    it under the terms of the GNU General Public License Version 2 as
**    published by the Free Software Foundation;
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program; if not, write to the Free Software
**    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
**
*/

#ifndef BILLMOVE_H
#define BILLMOVE_H

#include "vmath.h"

typedef VMvect myvec;


#define USE_ADV_BORDER
//#undef USE_ADV_BORDER
#define BILLMOVE_SINGLE_PRECISION
//#undef BILLMOVE_SINGLE_PRECISION


#ifndef USE_ADV_BORDER

#define  BorderType             OldBorderType
#define  BALL_WALL_DIST(b,w)    fabs(vec_mul(vec_diff(b.r,w.r),w.n))
#define  BALL_WALL_DIST_I(b,w)  vec_mul(vec_diff(b.r,w.r),w.n)

#else

#define  BorderType             AdvBorderType
#define  BALL_WALL_DIST_I(b,w)  ball_advborder_dist(&(b),&(w))
#define  BALL_WALL_DIST(b,w)    fabs(ball_advborder_dist(&(b),&(w)))

#endif

#define  BALL_BALL_DIST(b1,b2)  vec_abs(vec_diff(b1.r,b2.r))


#ifdef BILLMOVE_SINGLE_PRECISION
typedef float BMfloat;     /* floating point type for billard physics */
#else
typedef double BMfloat;    /* floating point type for billard physics */
#endif


typedef struct{
    BMfloat  m;           // mass                   [kg]
    BMfloat  I;           // massentraegheitsmom    [kg*m^2]
    BMfloat  d;           // diameter               [m]
    myvec    r;           // position               [m]
    myvec    v;           // speed                  [m/s]
    myvec    w;           // rotation speed and axe [rad./s] in table coords
    myvec    b[3];        // x,y,z base of ball in table coords
    int      nr;          // 0=white, ...
    int      in_game;     // ball in game
} BallType;

typedef struct{
    int        nr;
    BallType * ball;
} BallsType;

typedef struct{
    myvec r;             // pos
    myvec n;             // normal vector
} OldBorderType;

typedef struct{
    int   pnr;           // 3=triangle 2=line 1=point
    myvec r1;            // pos
    myvec r2;            // pos
    myvec r3;            // pos
    myvec n;             // normal vector
} AdvBorderType;

typedef struct{
    myvec pos;           // pos
    myvec aim;           // position to aim for ai-player
    BMfloat r;           // radius of hole (wall-coll free zone)
} HoleType;

typedef struct{
    int          nr;
    BorderType * border;
    int          holenr;
    HoleType *   hole;
} BordersType;


void BM_reset_move_info();
int BM_get_balls_out_half();
int BM_get_balls_out_full();
int BM_get_balls_out_total();
int BM_get_white_out();
int proceed_dt(BallsType *balls, BordersType *borders, BMfloat dt);

#ifdef USE_ADV_BORDER
BMfloat ball_advborder_dist( BallType *b, BorderType *w );
#endif

#endif  /* BILLMOVE_H */
